/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.ImageDiffer.ResultWriters;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import org.gfxtest.ImageDiffer.ComparisonResult;
import org.gfxtest.ImageDiffer.Configuration;

public class XmlWriter extends ResultWriter
{
    private static final String FILE_NAME = "results.xml";

    public XmlWriter(File outputDirectory) throws IOException
    {
        super(outputDirectory, FILE_NAME);
    }

    @Override
    public void printHeader() throws IOException
    {
        if (this.writer != null)
        {
            this.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            this.println("<visual-suite-result>");
        }
    }

    @Override
    public void printFooter() throws IOException
    {
        if (this.writer != null)
        {
            this.println("</visual-suite-result>");
        }
    }

    @Override
    public void printResultForOneImage(Configuration configuration, String imageFileName, BufferedImage[] sourceImages,
            ComparisonResult comparisonResult) throws IOException
    {
        String dirName = imageFileName.substring(0, imageFileName.lastIndexOf('.'));

        printComparisonResult(comparisonResult, dirName);
        printPerception(comparisonResult);
        this.println("        </pattern>");
        this.println("    </test>");
    }

    private void printComparisonResult(ComparisonResult comparisonResult, String dirName) throws IOException
    {
        boolean equalImages = comparisonResult.isEqualsImages();
        String result = equalImages ? "same" : "different";
        this.println("    <test id=\"%s\">", dirName);
        this.println("        <pattern id=\"%s\" result=\"%s\">", dirName, result);
        if (equalImages)
        {
            this.println("            <output/>");
        }
        else
        {
            this.println("            <output>%s</output>", dirName);
        }
    }

    @SuppressWarnings("boxing")
    private void printPerception(ComparisonResult cr) throws IOException
    {
        this.println("            <perception>");
        this.println("                <area width=\"%d\" height=\"%d\" />", cr.getAreaWidth(), cr.getAreaHeight());
        this.println("                <rectangle>");
        this.println("                    <vertex x=\"%d\" y=\"%d\" />", cr.getRectangleMin().x, cr.getRectangleMin().y);
        this.println("                    <vertex x=\"%d\" y=\"%d\" />", cr.getRectangleMax().x, cr.getRectangleMax().y);
        this.println("                </rectangle>");
        this.println("                <totalPixels>%d</totalPixels>", cr.getTotalPixels());
        this.println("                <globalDifferentPixels>%d</globalDifferentPixels>", cr.getDifferentPixels());
        this.println("                <unperceptibleDifferentPixels>%d</unperceptibleDifferentPixels>", cr.getSmallDifferences());
        this.println("                <samePixels>%d</samePixels>", cr.getEqualPixels());
        this.println("            </perception>");
    }

}
